%
%   This file is a help function of "Sshaper" developed by Anders
%    Blomqvist and Ryozo Nagamune. The theory behind the program is
%    presented in the technical report
%
%     "Sensitivity shaping with degree constraint by nonlinear
%     least-squares optimization"
%
%    (TRITA-MAT xxxxx, KTH, Stockholm, Sweden) and references
%    therin. Any questions can be sent to andersb@math.kth.se
%    and/or ryozo@me.berkeley.edu.
%
%   This function contains a specialized version of the
%    Gauss-Newton routine for the nonlinear least-squares problem
%    appearing in the tuning of degree constrained sensitivity 
%    functions.
%
function [schurpoly,resnorm,residual,exitflag] = ...
    ModifiedGN(fhandle,schurpoly,lb,ub,options,Sdesired,...
	       zvec,weightvec,zvecmat,testhandle,...
	       breakhandle,tmpflag);

%
%   NLSparameters already contain precomputed data.
%    Remove global variable? Anders Apr 7, 2004
%
global NLSparameters
global tmpdata

%
%   Parameters
%
maxiter = optimget(options,'MaxIter',100);
decreasefactor = 2;
eyefactor = 1/1000/length(NLSparameters.K);
printflag = optimget(options,'Display','off');
armijotol = 0.05;
steptol = 1e-10;
gradtol = 1e-6;

%
%   Optional printing
%
if strcmp(printflag,'iter')
  disp('        Modifieded Gauss-Newton Iterations')
  disp(' Iteration  Functional         mu     Norm of step')
end

%
%   Initialize
%
[F,Jac] = feval(fhandle,...
    schurpoly,Sdesired,zvec,weightvec,zvecmat);
f = F'*F/2;
grad = Jac'*F;
lambda = 1;
d = 1;

%
%   Main loop
%
mainiter = 0;
while  norm(grad) > gradtol & mainiter < maxiter & norm(lambda*d) > steptol
  mainiter = mainiter + 1;
    
  %
  %   Determine step
  %    Add small part of a identity matrix to get better scaled problem
  %
  H = Jac'*Jac;
  H = H + trace(H)*eyefactor*eye(size(H));
  grad = Jac'*F;
  d = - H \ grad;
  lambda = 1;
  
  %
  %   Check feasibility
  %
  feasiter = 0;
  while not(feval(testhandle,schurpoly + lambda*d'))...
	& feasiter < maxiter
    feasiter = feasiter + 1;
    lambda = lambda / decreasefactor;
  end
    
  %
  %   Try step & determine new residual vector and its Jacobian
  %
  [Fnew,Jacnew] = feval(fhandle,...
      schurpoly+lambda*d',Sdesired,zvec,weightvec,zvecmat);
  fnew = Fnew'*Fnew/2;
  
  %
  %   Line search conditions, currently only  Armijo
  %
  lineiter = 0;
  while fnew > f + armijotol*lambda*d'*grad & lineiter < maxiter
    lineiter = lineiter + 1;
    lambda = lambda / decreasefactor;
    
    feasiter = 0;
    while not(feval(testhandle,schurpoly + lambda*d'))...
	  & feasiter < maxiter
      feasiter = feasiter + 1;
      disp('Infeasible point! This shouldn''t happen unless very nonconvex')
      lambda = lambda / decreasefactor;
    end
    
    [Fnew,Jacnew] = feval(fhandle,...
	schurpoly+lambda*d',Sdesired,zvec,weightvec,zvecmat);
    fnew = Fnew'*Fnew/2;
  end

if 0
tmp = []; tmp2 = [];
xvec = -.1:.02:1.1;
[Fnewtmp,Jacnewtmp] = feval(...
    fhandle,schurpoly,Sdesired,zvec,weightvec, ...
    zvecmat);
for x = xvec
  [Fnewtmp,Jacnewtmp] = feval(...
      fhandle,schurpoly+x*d',Sdesired,zvec,weightvec,zvecmat);
  tmp(end+1) = Fnewtmp'*Fnewtmp/2;
  tmp2(end+1) = f+x*d'*grad + 1/2*x^2*d'*H*d;
end
figure(1), clf, plot(xvec,tmp,'k-',xvec,tmp2,'g:'), pause
end

  %
  %   Take accepted point
  %
  schurpoly = schurpoly + lambda*d';
  f = fnew;
  F = Fnew;
  Jac = Jacnew;

  %
  %   Optional printing
  %
  if strcmp(printflag,'iter')
    str1 = int2str(mainiter);
    str2 = num2str(f);
    str3 = num2str(lambda);
    str4 = num2str(norm(lambda*d));
    disp([blanks(7-length(str1)) str1...
	  blanks(15-length(str2)) str2...
	  blanks(14-length(str3)) str3...
	  blanks(14-length(str4)) str4])
  end
  
  %
  %   Temporary test to stop if we get too close to the boundary
  %
  if (nargin == 11 & feval(breakhandle,[1 schurpoly])) |...
	      (nargin > 11 & feval(breakhandle,schurpoly))
    break;
    disp('Broken')
  end

end

%
%   Return 
%
resnorm = f;
residual = F;
if mainiter < maxiter
  exitflag = 1;
  outmessage = 'Modified Gauss-Newton was sucessful';
else
  exitflag = 0;
  outmessage = [ 'Modified Gauss-Newton reached '...
      'maximum number of iterations'];
end

%
%   Display final
%
if strcmp(printflag,'iter') | strcmp(printflag,'final')
  disp(outmessage)
end

disp('**************************')
disp(['Gradient norm: ' num2str(norm(grad)) ])
