%
%   This file is a help function of "Sshaper" developed by Anders
%    Blomqvist and Ryozo Nagamune. The theory behind the program is
%    presented in the technical report
%
%     "Sensitivity shaping with degree constraint by nonlinear
%     least-squares optimization"
%
%    (TRITA-MAT xxxxx, KTH, Stockholm, Sweden) and references
%    therin. Any questions can be sent to andersb@math.kth.se
%    and/or ryozo@me.berkeley.edu.
%
%   This function contains a Levenberg-Marquardt routine specially
%    designed for the nonlinear least-squares problem appearing the
%    the tuning of degree constrained sensitivity functions.
%
function [poly,resnorm,residual,exitflag] = ...
    ModifiedLM(fhandle,poly,lb,ub,options,Sdesired,...
	       zvec,weightvec,zvecmat,testhandle,...
	       breakhandle,tmpflag);

%
%   NLSparameters already contain precomputed data.
%    Remove global variable? Anders Apr 7, 2004
%
global NLSparameters
global tmpdata

%
%   Parameters
%
maxiter = 2*optimget(options,'MaxIter',100);
decreasefactor = 2;
eyefactor = 1/1000/length(NLSparameters.K);
printflag = optimget(options,'Display','off');
steptol = 1e-10;
gradtol = 1e-6;

%
%   Optional printing
%
if strcmp(printflag,'iter')
  disp('      Modified Levenberg-Marquardt Iterations')
  disp(' Iteration  Functional    Delta       Norm of step')
end

%
%   Initialize
%
[F,Jac] = feval(fhandle,...
    poly,Sdesired,zvec,weightvec,zvecmat);

f = F'*F/2;
grad = Jac'*F;
H = Jac'*Jac;
Delta = 1;
mu = 1/4;
eta = 3/4;
d = 1;

fold = f;
sold = poly;

%
%   Main loop
%
mainiter = 0;
while  norm(grad) > gradtol * (1+f) & mainiter < maxiter ...
      & norm(d) > steptol
  mainiter = mainiter + 1;
  
  %
  %  Solve the trust-region sub-problem
  %
  d = trustdirect(H,grad,Delta);
  
  %
  %   Check feasibility and compute performance value
  %
  feasiter = 0;
  if not(feval(testhandle,poly + d'))
    lambda = 0;
  else
    [Fnew,Jacnew] = feval(...
	fhandle,poly+d',Sdesired,zvec,weightvec,zvecmat);
    lambda = (f - Fnew'*Fnew/2) / (-d'*grad - 1/2*d'*H*d);  
%
%   Test code by Anders April 21, 2004
%
if 0
  disp(norm(d))
  disp(d')
  disp(poly)
  tmp = []; tmp2 = []; xvec = -.1:0.05:1.2;
  [Fnewtmp,Jacnewtmp] = feval(...
      fhandle,poly,Sdesired,zvec,weightvec, ...
      zvecmat);
  for x = xvec
    [Fnewtmp,Jacnewtmp] = feval(...
	fhandle,poly+x*d',Sdesired,zvec,weightvec,zvecmat);
	tmp(end+1) = Fnewtmp'*Fnewtmp/2;
	tmp2(end+1) = f+x*d'*grad + 1/2*x^2*d'*H*d;
  end
  figure(1), clf, plot(xvec,tmp,'k-',xvec,tmp2,'g:'), pause       
end
  end

  %
  %   Update current point and parameters
  %
  if lambda <= mu
    Delta = Delta / 2;
  else
    [Fnew,Jacnew] = feval(...
	fhandle,poly+d',Sdesired,zvec,weightvec,zvecmat);
    poly = poly + d';
    grad = Jacnew'*Fnew;
    H = Jacnew'*Jacnew + 1e-8*eye(size(H));%  Change here if LM
                       % doesn't converge
    f = Fnew'*Fnew/2;
    fold(end+1) = f;
    sold(end+1,:) = poly;
    if lambda >= eta
      Delta = 2*Delta;
    end
    
  end
  
  %
  %   Optional printing
  %
  if strcmp(printflag,'iter')
    str1 = int2str(mainiter);
    str2 = num2str(f);
    str3 = num2str(Delta);
    str4 = num2str(norm(d));
    disp([blanks(7-length(str1)) str1...
	  blanks(15-length(str2)) str2...
	  blanks(14-length(str3)) str3...
	  blanks(14-length(str4)) str4])
  end

  %
  %   Temporary test to stop if we get too close to the boundary
  %
  if (nargin == 11 & feval(breakhandle,[1 poly])) |...
	      (nargin > 11 & feval(breakhandle,poly))
%    if nargin == 11
%      max(abs(roots([1 poly])))
%    else
%      max(abs(roots(poly)))
%    end      
    disp('Broken')
    break;
  end

end

%
%   Return 
%
resnorm = f;
residual = F;
if mainiter < maxiter
  exitflag = 1;
  outmessage = 'Modified Levenberg-Marquardt was sucessful';
else
  exitflag = 0;
  outmessage = [ 'Modified Levenberg-Marquardt reached '...
      'maximum number of iterations'];
end

%
%   Display final
%
if strcmp(printflag,'iter') | strcmp(printflag,'final')
  disp(outmessage)
end

disp('**************************')
disp(['Gradient norm: ' num2str(norm(grad)) ])

