%
%   This file is a help function of "Sshaper" developed by Anders
%    Blomqvist and Ryozo Nagamune. The theory behind the program is
%    presented in the technical report
%
%     "Sensitivity shaping with degree constraint by nonlinear
%     least-squares optimization"
%
%    (TRITA-MAT xxxxx, KTH, Stockholm, Sweden) and references
%    therin. Any questions can be sent to andersb@math.kth.se
%    and/or ryozo@me.berkeley.edu.
%
%   This function solves the nonlinear least squares problem
%
%      min || S(rho) - Shat ||^2 = F(rho)' F(rho)
%
%   over the set of Schur polynomials polynomials S.
%
%   Input:
%    K     - matrix relating the positive real numerator and
%             denominator
%    gamma - radius of range
%    Sdesired - desired frequency respons
%    zvec  - points for the desired frequency respons
%    alpha0 - initial point (maximum entropy solution)
%    rho0 - spectral numerator to initial point
%    weightvec - desired weight at the frequency points
%    printflag - printing to screen (1)
%    algflag - what algorithm to use
%    fixspeczeros - fixing some spectral zeros (only rho 
%                    parameterization)
%
%   Output
%    alpha    - denominator of optimal sensitivity function
%    schurpoly - corresponding spectral numerator
%    exitflag - diagnosis flag from optimization routine
%
function [alpha,schurpoly,exitflag,error] = NLSsolverred(...
    K,gamma,Sdesired,zvec,alpha0,rho0,weightvec,printflag,...
    algflag,fixspeczeros)

%
%   An ugly global variable that is updated along the optimization
%    routine 
%
try, clear NLSparameters, end
global NLSparameters

%
%   Default value if no weighting specified and normalized by
%    desired sensitivity function
%
if nargin < 7 | isempty(weightvec)
  weightvec = ones(size(Sdesired));
end
if any([ 3 4 7 8 ] == algflag)
  weightveccomp = weightvec ./ abs(Sdesired);
else
  weightvec = weightvec ./ abs(Sdesired);
  weightveccomp = weightvec;
end

options = optimset('Display','iter','Jacobian','on',...
		   'LargeScale','off','DerivativeCheck','off',...
		   'TolFun',1e-5,'TolX',1e-5);
if nargin >= 8 & not(isempty(printflag))
  options = optimset(options,'Display',printflag);
end

if nargin < 9
  algflag = 1;
end

%
%   Default no fixed spectral zeros
%
if nargin < 10
  fixspeczeros = [];
end

%
%   Check that initial point is normalized
%
if any(algflag == [3 4]) & rho0(1) ~= 1
  disp('Not normalized spectral parametrization')
  alpha0 = alpha0 / rho0(1);
  rho0 = rho0 / rho0(1);
elseif any(algflag == [7 8]) & alpha0(1) ~= 1
  disp('Not normalized direct parameterization')
  rho0 = rho0 / alpha0(1);
  alpha0 = alpha0 / alpha0(1);
end  

%
%   Construct the parameter structure
%
NLSparameters.r = poly(fixspeczeros);
NLSparameters.T = toeplitz(...
    [NLSparameters.r zeros(1,length(alpha0)-length(NLSparameters.r))],...
    NLSparameters.r(1)*eye(1,length(alpha0)-length(NLSparameters.r)+1));
NLSparameters.alpha0 = alpha0(:);
NLSparameters.schurpoly0 = rho0(:)';
NLSparameters.disspoly0 = GetDissPoly(...
    conv(NLSparameters.schurpoly0,NLSparameters.r));
NLSparameters.K = K;
NLSparameters.gIK = gamma*eye(size(K)) + K;
NLSparameters.gIKinv = inv(NLSparameters.gIK);
NLSparameters.Khat = (gamma*eye(size(K)) - K)...
    * NLSparameters.gIKinv;
% For direct parametrization
NLSparameters.gamma = gamma;

%
%   Normalize in disspation polynomial parameterization
%
if any(algflag == [11 12]) & NLSparameters.disspoly0(1) ~= 1
  NLSparameters.alpha0 = NLSparameters.alpha0 / ...
      sqrt(NLSparameters.disspoly0(1));
  NLSparameters.schurpoly0 = NLSparameters.schurpoly0 / ...
      sqrt(NLSparameters.disspoly0(1));
  NLSparameters.disspoly0 = NLSparameters.disspoly0 / ...
      NLSparameters.disspoly0(1);
end

%
%   Initialize local parameters
%
schurpoly = NLSparameters.schurpoly0;%,pause
disspoly = NLSparameters.disspoly0;%,pause
Khat = NLSparameters.Khat;  

%
%   Compute powers of zvec
%
n = size(K,1);
zvecmat(:,n) = ones(size(zvec));
for k = 1:n-1
  zvecmat(:,n-k) = zvec.^k;
end

%
%   Solve nonlinear least-squares problem.
%
switch algflag
 case 1
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedLM(@NLSsensitivityred,schurpoly(2:end),[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsSchurnorm,...
		 @IsBoundary);
  schurpoly = [ 1 schurpoly ];
 case 2
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedGN(@NLSsensitivityred,schurpoly(2:end),[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsSchurnorm,...
		 @IsBoundary);
  schurpoly = [ 1 schurpoly ];
 case 3
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedLM(@NLSsensitivityredpoly,schurpoly(2:end),[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsSchurnorm,...
		 @IsBoundary);
  schurpoly = [ 1 schurpoly ];
 case 4
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedGN(@NLSsensitivityredpoly,schurpoly(2:end),[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsSchurnorm,...
		 @IsBoundary);
  schurpoly = [ 1 schurpoly ];
 case 5
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedLM(@NLSsensitivitydirect,alpha0(2:end)',[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@ ...
		 IsBoundedRealNLSnorm);
 case 6
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedGN(@NLSsensitivitydirect,alpha0(2:end)',[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsBoundedRealNLSnorm);
 case 7
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedLM(@NLSsensitivitydirectpoly,alpha0(2:end)',[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsBoundedRealNLSnorm);
 case 8
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedGN(@NLSsensitivitydirectpoly,alpha0(2:end)',[],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsBoundedRealNLSnorm);
 case 9
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedLM(@NLSsensitivityred0,schurpoly,[],[],options,Sdesired,...
		 zvec,weightvec,zvecmat,@IsSchur,...
		 @IsBoundary,1);
  schurpoly = [ 1 schurpoly ];
 case 10
  [schurpoly,resnorm,residual,exitflag] = ...
      ModifiedGN(@NLSsensitivityred0,schurpoly,[],[],options,Sdesired,...
		 zvec,weightvec,zvecmat,@IsSchur,...
		 @IsBoundary,1);
  schurpoly = [ 1 schurpoly ];
 case 11
  if length(fixspeczeros) > 0
    disp('Fixed spectral zeros not supported for this solver'), pause
  end
  [disspoly,resnorm,residual,exitflag] = ...
      ModifiedLM(@NLSsensitivityreddiss,disspoly(2:end)'/disspoly(1),...
		 [],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsDissPolynorm);
  schurpoly = wilson_fix([1 2*disspoly],1e-10);
  disspoly = [ 1 disspoly ];
 case 12
  if length(fixspeczeros) > 0
    disp('Fixed spectral zeros not supported for this solver'), pause
  end
  [disspoly,resnorm,residual,exitflag] = ...
      ModifiedGN(@NLSsensitivityreddiss,disspoly(2:end)'/disspoly(1),...
		 [],[],options,...
		 Sdesired,zvec,weightvec,zvecmat,@IsDissPolynorm);
  schurpoly = wilson_fix([1 2*disspoly],1e-10);
  disspoly = [ 1 disspoly ];
end

%
%   Display spectral zeros
%
disp(['Maximimal modulus of spectral zeros: '...
	num2str(max(abs(roots(schurpoly)))) ])
disp('**************************')



%
%   Return the solution
%
alpha = NLSparameters.alpha0;

%
%   Return an error
%
F = (polyval(K*alpha,zvec)./polyval(alpha,zvec) - Sdesired)...
    .* weightveccomp;
F = [ real(F); imag(F) ];
error = F'*F;
