%
%   This file is a help function of "Sshaper" developed by Anders
%    Blomqvist and Ryozo Nagamune. The theory behind the program is
%    presented in the technical report
%
%     "Sensitivity shaping with degree constraint by nonlinear
%     least-squares optimization"
%
%    (TRITA-MAT xxxxx, KTH, Stockholm, Sweden) and references
%    therin. Any questions can be sent to andersb@math.kth.se
%    and/or ryozo@me.berkeley.edu.
%
%   This function determines the interpolation constraints that 
%    ensures internal stability of a SISO plant - continuous or
%    discrete time. 
%
%   Input:
%    sys - plant as an LTI system
%    kappa - scaling facor of domain unit circle
%    gamma - upper limit of |S|
%    Creldegree - required relative degree of the controller
%    Cintegrators - intergrators in C
%    fixpoints - additional interpolation points
%    fixval - additional interpolation values
%
%   Output:
%    Aout,Bout - basis for interpolation problem
%    Kout      - tranformation matrix for interpolation problem
%    alphaout  - maximum entropy soluiton, optional output
%    sigmaout  - corresponding spectral numerator for its 
%                 positive real function
%    PCintegrators - number in integratots in continuous time
%
function [Aout,Bout,Kout,alphaout,rhoout,PCintegrators,exitflag] = ...
    SISO2K(sys,kappa,gamma,Creldegree,Cintegrators,...
	      fixpoint,fixval,printflag)

%
%   Exit flag default ok
%
exitflag = 1;

%
%   Treat additional input
%
if nargin <= 5
  fixpoint = [];
  fixval = [];
end

%
%  Interpolation matrices for additional constraints
%
[Aadd,Badd,Wadd] = GetRealBasis(fixpoint,fixval);

%
%   Determine the interpolation points
%
ustpoles = pole(sys);
nmpzeros = zero(sys);
reldegree = length(ustpoles)-length(nmpzeros)+Creldegree;
if isct(sys)
  %
  %   Continuous time plant
  %
  nmpzeros = [ -ones(reldegree,1);...
	       (1-nmpzeros(find(real(nmpzeros)>=0)))./...
	       (1+nmpzeros(find(real(nmpzeros)>=0))) ];
  ustpoles = [ ones(Cintegrators,1);...
	      (1-ustpoles(find(real(ustpoles)>=0)))./...
	      (1+ustpoles(find(real(ustpoles)>=0))) ];
  PCintegrators = length(find(abs(ustpoles-1)<1e-10));
else
  %
  %   Discrete time plant
  %
  nmpzeros = [ zeros(reldegree,1);...
	       1./nmpzeros(find(abs(nmpzeros)>=1)) ];
  ustpoles = [ ones(Cintegrators,1);...
	       1./ustpoles(find(abs(ustpoles)>=1)) ];
  PCintegrators = 0;
end

%
%   Constuct basis and interdata matrix
%
[A,B] = GetRealBasis(ustpoles);
W = zeros(size(A));
[Az,Bz] = GetRealBasis(nmpzeros);
A = blkdiag(A,Az,Aadd);
B = [B; Bz; Badd];
W = blkdiag(W,eye(size(Az)),Wadd); 

%
%   Get transformation matrix
%
Gamma = ctrb(A,B);
K = Gamma \ W * Gamma;

%
%   Give output
%
Aout = A;
Bout = B;
Kout = K;

%
%   If asked for, determine maximum entropy solution
%
if nargout > 3
  %
  %   Transform problem so that the interpolation points are in the
  %    interior of the unit disc. This give an approximate solution.
  %
  flag = 1;
  kappa = 1.01;
  while flag
    %
    %   Try with current kappa
    %
    intvec = [ ustpoles; nmpzeros; fixpoint(:)] / kappa;
    ustpolestmp = ustpoles / kappa;
    nmpzerostmp = nmpzeros / kappa;
    fixpointtmp = fixpoint / kappa;
    
    %
    %   Check if any real interpolation point is zero - otherwise
    %    move one there.
    % 
    if isempty(intvec)
      disp('No interpolation conditions necessary')
      return
    else
      intvec = intvec(find(imag(intvec) == 0));
      if isempty(intvec)
	disp('Only imaginary interpolation conditions')
	return
      elseif all(intvec ~= 0)
	z0 = intvec(find(abs(intvec) == min(abs(intvec))));
	trans = GetBlTrans(1,-z0(1),-conj(z0(1)),1);
	ustpolestmp = MoveInterpoints(trans,ustpolestmp);
	nmpzerostmp = MoveInterpoints(trans,nmpzerostmp);
	fixpointtmp = MoveInterpoints(trans,fixpointtmp);
      else
	trans = GetBlTrans(1,0,0,1);
      end
    end
    
    %
    %   Compute corresponding basis matrices
    %
    [Aadd,Badd] = GetRealBasis(fixpointtmp);
    [A,B] = GetRealBasis(ustpolestmp);
    W = zeros(size(A));
    [Az,Bz] = GetRealBasis(nmpzerostmp);
    A = blkdiag(A,Az,Aadd);
    B = [B; Bz; Badd];
    W = blkdiag(W,eye(size(Az)),Wadd);
    
    %
    %   Get transformation matrix
    %
    Gamma = ctrb(A,B);
    K = Gamma \ W * Gamma;
    
    %
    %   Compute transformation matrix for PR case
    %
    Khat = (gamma*eye(size(K)) - K) /...
	   (gamma*eye(size(K)) + K);
    
    %
    %   Interpolation data for PR case
    %
    What = Gamma * Khat / Gamma;
    
    %
    %   Compute Pick matrix
    %
    E = dlyap(A,B*B');
    Phat = What*E + E*What';

    %
    %   Check positivity and well-scaledness of the Pick matrix
    %
    mineig = min(eig(Phat));
    if mineig <= 0
      disp('Pick condition not satisfied! Give new indata!')
      disp([ '  Smallest eigenvalue: ' num2str(mineig) ])
      exitflag = 0;
      Phat = eye(size(Phat));
      break
    elseif mineig <= 1e-8
      disp(min(eig(Phat))), pause
      kappa = kappa * 1.05;
    else
      flag = 0;
    end
  end
  
  %
  %   Compute maximum entropy (ME) solution
  %
  PhatB = Phat \ B;
  CME = 1/sqrt(B'*PhatB) * PhatB;
  sysME = inv(ss(A,B,CME'*A,CME'*B,-1));
  
  %
  %   Transform back to original domain
  %
  [rhoout,alphaout] = TransBack(sysME,trans,Kout,gamma,kappa);

end


%
%   Help function to generate the Input-to-State transfer matrices
%    describing the interpolation points.
%
function [A,B,W] = GetRealBasis(zvec,wvec)

%
%   Determine initialize basis and give tolerance
%
A = [];
B = [];
tol = 1e-6; 
if nargin > 1, W = []; end

%
%   Main loop for determining constructing the basis from one pole
%    at the time.
%
while not(isempty(zvec))
  idx = find(abs(zvec-zvec(1)) < 2*tol);
  nidx = length(idx);
  if abs(imag(zvec(1))) < tol
    %
    %   Real case
    %
    Aadd = diag(real(zvec(1))*ones(1,nidx)) + diag(ones(1,nidx-1),-1);
    Badd = eye(nidx,1);
    Wadd = [];
    if nargin > 1
       Wadd = toeplitz(wvec(idx),[wvec(1),zeros(1,nidx-1)]);
    end
  else
    %
    %   Complex case
    %
    idxconj = find(abs(zvec-conj(zvec(1))) < 2*tol);

    if length(idxconj) ~= nidx
      disp(['In GetRealBasis: Poles doesn''t seem complex' ...
	    ' conjugate'])
    end

    Aadd = [ real(zvec(1)) -imag(zvec(1)); imag(zvec(1)) real(zvec(1)) ];
    Badd = eye(2,1);
    Wadd = [];
    if nargin > 1
      Wadd(:,:,1) = [ real(wvec(1)) -imag(wvec(1));...
		      imag(wvec(1)) real(wvec(1)) ];
    end

    if nidx > 1
      %
      %   Multiple case
      %
      for k = 2:nidx
	Aadd = blkdiag(Aadd,Aadd(1:2,1:2));
	if nargin > 1
	  Wadd(:,:,k) = ...
	      [ real(wvec(idx(k))) -imag(wvec(idx(k)));...
		imag(wvec(idx(k))) real(wvec(idx(k))) ];
	end
      end
      Aadd = Aadd + diag(ones((nidx-1)*2,1),-2);
      Badd(3:2*nidx) = 0;
      if nargin > 1
	Wtmp = Wadd;
	Wtmp(:,:,2:end) = 0;
	Wadd = BlkToeplitz(Wadd,Wtmp);
      end
    end
    idx(end+1:end+nidx) = idxconj;
  end
  A = blkdiag(A,Aadd);
  B = [ B; Badd ];
  zvec(idx) = [];
  if nargin > 1
    W = blkdiag(W,Wadd);
    wvec(idx) = [];
  end
end


%
%   Local help function for transforming back
%
function [rho,alpha] = TransBack(sys,trans,K,gamma,kappa)

%
%   Transform back to original domain
%
sys = BilinearDomain(trans,ss(sys));

%
%   Give output
%
sys = minreal(tf(sys));
[rho,alpha] = tfdata(sys,'v');
if length(alpha) < size(K,1)
  n = size(K,1) - length(alpha);
  alpha = [ alpha zeros(1,n) ];
  rho = [ rho zeros(1,n) ];
end
alpha = alpha/rho(1);
rho = rho/rho(1);
alpha = ((gamma*eye(size(K)) + K) \ alpha')';

%
%   Check that solution is real
%
if sum(abs(imag([rho alpha]))) > 1e-3
  disp('The solution is not real')
  disp(rho)
  disp(alpha)
else
  alpha = real(alpha(:));
  rho = real(rho(:));
end
  
%
%  Transform to original formulation
%
kappavec = ((kappa + 1e-4).^(size(K,1)-1:-1:0)');
alpha = alpha.* kappavec;

%
%  Compute exact spectral numerator
%
alpha0 = (gamma*eye(size(K)) + K) * alpha(:);
beta0 = (gamma*eye(size(K)) - K) * alpha(:);
d0 = GetS(alpha0)*beta0(:);
rho = wilson_fix([d0(1) 2*d0(2:end)'],1e-10);