%
%   This is the main file for the Sshaper program
%
%   This script provide an interface to the nonlinear least-squares
%    optimization approach to sensitivity shaping using the degree
%    constrained Nevanlinna-Pick approach.
%
%   This file is a part of "Sshaper" developed by Anders Blomqvist
%    and Ryozo Nagamune. The theory behind the program is presented
%    in the technical report
%
%     "Sensitivity shaping with degree constraint by nonlinear
%     least-squares optimization"
%
%    (TRITA-MAT xxxxx, KTH, Stockholm, Sweden) and references
%    therin. Any questions can be sent to andersb@math.kth.se
%    and/or ryozo@me.berkeley.edu.
%
addpath homotopy2.0/
addpath helpfiles/
format compact
warning off

%
%   Main loop
%
mainflag = 0;
while not(mainflag)
  %
  %   Initialize variables if cleared.
  %
  try
    plotparameters;
  catch
    plotparameters = InitializePlotparameters(1);
  end

  try
    printparameters;
  catch
    printparameters = InitializePrintparameters(2.5);
  end

  try sysP;          catch sysP = tf([]);      end
  try Creldegree;    catch Creldegree = 0;     end
  try Cintegrators;  catch Cintegrators = 0;   end
  try isreal(theta); catch theta = [];         end
  try Sdesired;      catch Sdesired = [];      end
  try isreal(gamma); catch gamma = 2;          end
  try weight;        catch weight = [];        end
  try spectralzeros; catch spectralzeros = []; end
  try peakfreq;      catch peakfreq = [];      end
  try sharpness;     catch sharpness = [];     end
  try rho;           catch rho = [];           end
  try xspec;
  catch xspec.freq = []; xspec.sharpness = []; ...
	xspec.mag = []; xspec.phase = []; 
  end
  try xspectemp;    catch xspectemp.freq = []; end

  mainoptions = { 'Input Data Menu',...
		  'Design Controller',...
		  'Load Controller for Comparison',...
		  'Plot Menu',...
		  'Print Menu',...
		  'Load Design',...
		  'Save Design',...
		  'Display Variables',...
		  'Clear Variables',...
		  'Close Figures',... 
		  'Exit' };
  chosenmainoption = DrawMenu('Main Menu',mainoptions);
      
  switch chosenmainoption
    
   %
   %   Load and modify the input data 
   %
   case 'Input Data Menu'
    inputoptions = { 'Load Benchmark Example',...
		     'Plant',...
		     'Controller Requirements',...
		     'Desired Frequency Response and Weight',...
		     'Other Design Parameters',...
		     'Return to Main Menu' };
    
    %
    %   Input loop
    %
    inputflag = 0;
    while not(inputflag)
      choseninputoption = DrawMenu('Input Data Menu',inputoptions);
      switch choseninputoption
       
       %
       %   Load standard examples - hard coded
       %
       case 'Load Benchmark Example'
	benchmarkoptions = { 'Flexible Beam',...
		    'Slide Drive Control',...
		    'Return to Input Data Menu' };
	chosenbenchmarkoption = DrawMenu(['Choose Benchmark' ...
		    ' Example'],benchmarkoptions);
	switch chosenbenchmarkoption
	 case 'Flexible Beam'
	  load DesignBenchmark/ExFlexBeam.mat
	  printparameters.examplename = 'FlexBeam';
	 case 'Slide Drive Control'
	  load DesignBenchmark/ExSlide.mat
	  printparameters.examplename = 'Slide';
	 case 'Return to Input Data Menu'
	end
	 
       %
       %   Load or modify the plant
       %
       case 'Plant'
	Pvec = { '  Plant: ' };
	Pflag = 1;
	while Pflag
	  Pvaluevec = { sysP };
	  flag = DrawCurrent(Pvec,Pvaluevec);
	  if flag
	    flag = input('Do you want to load it from a file? (y)','s');
 	    if strcmp(flag,'y') | isempty(flag)       
	      files = dir;
	      dirname = {};
	      for k = 1:length(files)
		if files(k).isdir
		  dirname{end+1} = files(k).name;
		end              
	      end          
	      dirname = DrawMenu('Available directories',dirname);
	      files = dir([dirname '/*.mat']);
	      availfiles = {};
	      for k = 1:length(files)
		availfiles{k} = files(k).name;
	      end
	      availfiles{end+1} = 'No input';
	      chosenfile = DrawMenu('Available files in directory',availfiles);
	      if not(strcmp(chosenfile,'No input'))
		load([dirname '/' chosenfile],'sysP','theta','Sdesired') 
	      end
	    else
	      Ts = input(...
		  'Give sampling time, 0 for continuous plants. '); 
	      numP = input('Give numerator ');
	      denP = input('Give denominator ');
	      if isempty(numP) ~= 1 & isempty(denP) ~= 1
		sysP = tf(numP,denP,Ts);
	      end
	    end
	  else
	    Pflag = 0;
	  end
	end
	
       %
       %   Modify the requirements of the controller
       %
       case 'Controller Requirements'
	Cvec =  { '  relative degree of controller: ',...
		  '  number of integrators in the controller: '};
	Cflag = 1;
	while Cflag
	  Cvaluevec = { Creldegree Cintegrators };		   
	  flag = DrawCurrent(Cvec,Cvaluevec);
	  if flag
	    Creldegree = NumericalInput(...
		'Give desired relative degree of the controller:',...
		Creldegree);
	    Cintegrators = NumericalInput(...
		'Give desired number of controller integrators:',...
		Cintegrators);
	  else
	    Cflag = 0;
	  end
	end
       
       %
       %   Define the gridding and a desired sensitivity function
       %
       case 'Desired Frequency Response and Weight'
	Dvec = { '  displayed in Figure ' };
	Dvaluevec = { 1 };
	Dflag = 1; 

	%
	%   Loop for modifying the desired sensitivity funtion
	%    The variable Sd contains all temporary variable and
        %    will be erased after leaving the loop.
	%
	while Dflag
	  %
	  %   Replot in Figure 1 (hard coded)
	  %
	  if size(weight) ~= size(theta)
	    weight = ones(size(theta));
	  end
	  Sd = PlotInteractive(...
	      theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
	  flag = DrawCurrent(Dvec,Dvaluevec); 
	  if flag
	    options = { 'Load from file',...
			'Give condensed',...
			'Give manually',...
		        'Keep current' };
	    chosenoption = DrawMenu('Gridding',options);
	    switch chosenoption
	     
	     %
	     %   Gridding from file
	     %
	     case 'Load from file'
	      filename = input('Give the file name? ','s');
	      load(filename)
	      try
		if size(theta) ~= size(weight)
		  weight = ones(size(theta));
		end
	      catch
		  weight = ones(size(theta));
	      end		
	     
	     %
	     %   Gridding by lower, upper and number of points
	     %
	     case 'Give condensed'
	      %
	      %   Define default values (fix)
	      %
	      lower = 1e-3;
	      upper = 1e3;
	      n = 100;
	      lower = NumericalInput('Give lower limit (rad/sec):',lower);
	      upper = NumericalInput('Give upper limit (rad/sec):',upper);
	      n = NumericalInput('Give number of points:',n);
	      if isdt(sysP) 
		theta = logspace(log10(lower*sysP.Ts),log10(upper*sysP.Ts),n);
	      else
		theta = logspace(log10(lower),log10(upper),n);
	      end
          
	     %
	     %    Gridding manually
	     %
	    case 'Give manually'
	     theta = input('Give gridding vector ');
	    end
	    %
	    %   Make column vector
	    %
	    theta = theta(:);
	    options = { 'Load from file',...
			'Give ideal',...
			'Use previously designed S',...
			'Give manually',...
			'Give manually in region',...
			'Give manually by window',...
			'Give Interactively in Plot',...
			'Keep current' };
	    chosenoption = DrawMenu('Desired Sensitivity and Weight',options);
	    switch chosenoption
	      
	     %
	     %    Sdesired from file
	     %
	     case 'Load from file'
	      filename = input('Give the file name? ','s');
	      load(filename)
	       
	     %
	     %    Sdesired by rational function
	     %
	     case 'Give ideal'
	      numSid = NumericalInput('Give numerator:',[1 1.2 0]);
	      denSid = NumericalInput('Give denominator:',[1 1.2 1]);
	      if sysP.Ts ~= 0 
		Sdesired = polyval(numSid,exp(i*theta)) ./ ...
		    polyval(denSid,exp(i*theta));
	      else
		Sdesired = polyval(numSid,i*theta) ./ ...
		    polyval(denSid,i*theta);
	      end

	     %
	     %    S designed in the previous iteration
	     %
	     case 'Use previously designed S'
	      if isempty(sysS)
		disp('There is no designed S.'), pause
	      else
		[numSid,denSid] = tfdata(sysS,'v');
		if sysP.Ts ~= 0
		  Sdesired = polyval(numSid,exp(i*theta)) ./ ...
		      polyval(denSid,exp(i*theta));
		else
		  Sdesired = polyval(numSid,i*theta) ./ ...
		      polyval(denSid,i*theta);
		end
	      end
	      
	     %
	     %    Sdesired & weight by manual input
	     %
	     case 'Give manually'
	      Sdesired = NumericalInput(['Give desired sensitivity (length ',...
				num2str(length(theta)) '): '],Sdesired);
	      weight = NumericalInput(['Give weight (length ',...
				num2str(length(theta)) '): '], ...
				      weight);
	      Sdesired = Sdesired(:);
	      weight = weight(:);

	     %
	     %    Sdesired & weight by manual in region
	     %
	     case 'Give manually in region'
	      if sysP.Ts ~= 0
		thetatmp = theta / sysP.Ts;
	      else
		thetatmp = theta;
	      end
	      
	      Sd.weightmod = weight;
	      Sd.weightnew = weight;
	      Sd.Sdesiredmod = Sdesired;
	      Sd.Sdesirednew = Sdesired;
	      Mflag = 1;
	      while Mflag
		disp(['In which frequency region do you want to change' ...
		      ' the desired sensitivity?'])
		lower = NumericalInput('Give lower limit',thetatmp(1));
		upper = NumericalInput('Give upper limit', ...
				       thetatmp(end));
		idx = find(thetatmp >= lower & thetatmp <= upper);
		magnow = 20*log10(abs(Sd.Sdesiredmod(idx)));
		phasenow = angle(Sd.Sdesiredmod(idx));
		weightnow = Sd.weightmod(idx); 
		disp(['Index   Frequency    Magnitude(dB)    Phase' ...
		      '    Weight'])
		for k = 1:length(idx)
		  Sd.str1 = num2str(thetatmp(idx(k)),4);
		  Sd.str2 = num2str(magnow(k),5);
		  Sd.str3 = num2str(phasenow(k),5);
		  Sd.str4 = num2str(weightnow(k),5);
		  disp([ blanks(2) int2str(idx(k))...
			 blanks(13-length(Sd.str1)) Sd.str1...
			 blanks(13-length(Sd.str2)) Sd.str2...
			 blanks(13-length(Sd.str3)) Sd.str3...
			 blanks(10-length(Sd.str4)) Sd.str4 ])
		end
		idxstr = num2str(length(idx));
		% Magnitude
		disp(['Your current magnitude in the region is available' ...
		      ' in ''default'' '])
		flag = 1;
		while flag
		  magnowtmp = NumericalInput(['Give new magnitude (dB) (Length: ' ...
		    idxstr ')'],magnow);
		  if length(magnowtmp) == 1,
		    magnowtmp = magnowtmp*ones(1,length(idx));
		  end        
		  if length(idx) == length(magnowtmp)
		    flag = 0;
		    magnow = magnowtmp;
		    clear magnowtmp
		  else
		    disp(['Length of the vector must be ' idxstr '. Try again.'])
		  end
		end
		% Phase
		disp(['Your current phase in the region is available' ...
		      ' in ''default'' '])
		flag = 1;
		while flag
		  phasenowtmp = NumericalInput(['Give new phase (rad/sec) (Length: ' ...
		    idxstr ')'],phasenow);
		  if length(phasenowtmp) == 1,
		    phasenowtmp = phasenowtmp*ones(1,length(idx));
		  end        
		  if length(idx) == length(phasenowtmp)
		    flag = 0;
		    phasenow = phasenowtmp;
		    clear phasenowtmp
		  else
		    disp(['Length of the vector must be ' idxstr '. Try again.'])
		  end
		end
		% Weight
		disp(['Your current weight in the region is available' ...
		      ' in ''default'' '])
		flag = 1;
		while flag
		  weightnowtmp = NumericalInput(['Give new weight (Length: '...
		    idxstr ')'],weightnow);
		  if length(weightnowtmp) == 1,
		    weightnowtmp = weightnowtmp*ones(1,length(idx));
		  end          
		  if length(idx) == length(weightnowtmp)
		    flag = 0;
		    weightnow = weightnowtmp;
		    clear weightnowtmp
		  else
		    disp(['Length of the vector must be ' idxstr '. Try again.'])
		  end
		end        
                Sd.Sdesirednew(idx) = 10.^(magnow(:)/20).*exp(i*phasenow(:));
		Sd.weightnew(idx) = weightnow(:);
		Sd = PlotDesired(Sd,theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
		flag = input('Do you want to use your new sensitivity and weight? (y)',...
			     's');
		if strcmp(flag,'y') | isempty(flag)
		  Sd.Sdesiredmod = Sd.Sdesirednew;
		  Sd.weightmod = Sd.weightnew;
		end
		flag = input('Do you want to continue modifying? (y)',...
			     's');
		if not(strcmp(flag,'y') | isempty(flag))
		  Mflag = 0;
		end
	      end
	      flag = input('Do you really want to use your new sensitivity? (y)',...
			     's');
	      if strcmp(flag,'y') | isempty(flag)
		Sdesired = Sd.Sdesiredmod;
		weight = Sd.weightmod;
	      end   	      
	     %
	     %    Sdesired & weight by manual window
	     %
	     case 'Give manually by window'
	      Sd.weightmod = weight;
	      Sd.Sdesiredmod = Sdesired;
	      Mflag = 1;
	      if sysP.Ts ~= 0
		thetatmp = theta / sysP.Ts;
	      else
		thetatmp = theta;
	      end
	      while Mflag
		center = NumericalInput(['In what frequency region do you want ' ...
		    'to center your change?'],thetatmp(1));
		idx = find(thetatmp <= center);
		if isempty(idx)
		  idx = 1;
		else
		  idx = idx(end);
		end
		width = NumericalInput('Give the width of the window',...
				       length(thetatmp)/20);
		magnow = NumericalInput('Give new magnitude at center (dB)',...
					20*log10(abs(Sd.Sdesiredmod(idx))));
		phasenow = NumericalInput('Give new phase at center',...
					  angle(Sd.Sdesiredmod(idx)));
		weightnow = NumericalInput('Give new weight at center',...
					  Sd.weightmod(idx));

		%
		%   Modify magnitude
		%
		Sd.factor = 1 + (10^(magnow(:)/20) - abs(Sd.Sdesiredmod(idx)))/...
		    abs(Sd.Sdesiredmod(idx))...
		    *exp(-((((1:length(theta))-idx)/...
			    width).^2/2));
		Sd.Sdesirednew = Sd.Sdesiredmod.*Sd.factor(:);
		%
		%   Modify phase
		%
		Sd.factor = (phasenow(:) - angle(Sd.Sdesiredmod(idx)))...
		    *exp(-((((1:length(thetatmp))-idx)/...
			    width).^2/2));
		Sd.Sdesirednew = abs(Sd.Sdesirednew).*...
		    exp(i*(Sd.factor(:)+angle(Sd.Sdesirednew)));
		%
		%   Modify weight
		%
		Sd.factor = 1 + (weightnow(:) - Sd.weightmod(idx))/Sd.weightmod(idx)...
		    *exp(-((((1:length(thetatmp))-idx)/...
			    width).^2/2));
		Sd.weightnew = Sd.weightmod.*Sd.factor(:);
		%
		%   Check if we want to keep it
		%
		Sd = PlotDesired(Sd,theta,Sdesired,weight,plotparameters,...
				 sysP.Ts,gamma,xspec);
		flag = input(['Do you want to use your new sensitivity '...
			      'and weight? (y)'],'s');
		if strcmp(flag,'y') | isempty(flag)
		  Sd.Sdesiredmod = Sd.Sdesirednew;
		  Sd.weightmod = Sd.weightnew;
		else
		  Sd.Sdesirednew = Sd.Sdesiredmod;
		  Sd.weightnew = Sd.weightmod;
		  Sd = PlotDesired(Sd,theta,Sdesired,weight,plotparameters,...
				   sysP.Ts,gamma,xspec);
		end
		flag = input('Do you want to continue modifying? (y)',...
			     's');
		if not(strcmp(flag,'y') | isempty(flag))
		  Mflag = 0;
		end
	      end
	      flag = input(['Do you really want to use your new sensitivity '...
			    '& weight? (y)'],'s');
	      if strcmp(flag,'y') | isempty(flag)
		Sdesired = Sd.Sdesiredmod;
		weight = Sd.weightmod;
	      end  

	     %
	     %    Sdesired interactively in plot
	     %
	     case 'Give Interactively in Plot'
	      disp('Click in Figure 1 to give a desired value')
	      disp('Finish by clicking on the right button')
	      Sd.weightmod = weight;
	      Sd.Sdesiredmod = Sdesired;
	      [x,y,interactiveflag] = ginput(1);
	      if sysP.Ts ~= 0
		thetatmp = theta / sysP.Ts;
	      else
		thetatmp = theta;
	      end
	      while interactiveflag == 1
		idx = find(thetatmp < x);
		if isempty(idx)
		  idx = 1;
		else
		  idx = idx(end);
		end
		if gca == Sd.subplot1
		  %
		  %   Modify magnitude
		  %
		  Sd.factor = 1 + (10^(y/20) - abs(Sd.Sdesiredmod(idx)))/...
		      abs(Sd.Sdesiredmod(idx))...
		      *exp(-((((1:length(thetatmp))-idx)/...
			      (length(thetatmp)/20)).^2/2));
		  Sd.Sdesirednew = Sd.Sdesiredmod.*Sd.factor(:);
		  Sd.weightnew = Sd.weightmod;
		elseif gca == Sd.subplot2
		  %
		  %   Modify phase
		  %
		  Sd.factor = (y*pi/180 - angle(Sd.Sdesiredmod(idx)))...
			   *exp(-((((1:length(thetatmp))-idx)/...
				   (length(thetatmp)/20)).^2/2));
		  Sd.Sdesirednew = abs(Sd.Sdesiredmod).*...
		      exp(i*(Sd.factor(:)+angle(Sd.Sdesiredmod)));
		  Sd.weightnew = Sd.weightmod;
		elseif gca == Sd.subplot3
		  %
		  %   Modify weight
		  %
		  Sd.Sdesirednew = Sd.Sdesiredmod;
		  Sd.factor = 1 + (y - Sd.weightmod(idx))/Sd.weightmod(idx)...
			   *exp(-((((1:length(thetatmp))-idx)/...
				   (length(thetatmp)/20)).^2/2)); 
		  Sd.weightnew = Sd.weightmod.*Sd.factor(:);
		end
		Sd = PlotDesired(Sd,theta,Sdesired,weight,plotparameters,...
				 sysP.Ts,gamma,xspec);
		flag = input('Do you want to use the new sensitivity? (y)',...
			     's');
		if strcmp(flag,'y') | isempty(flag)
		  Sd.Sdesiredmod = Sd.Sdesirednew;
		  Sd.weightmod = Sd.weightnew;
		else
		  Sd.Sdesirednew = Sd.Sdesiredmod;
		  Sd.weightnew = Sd.weightmod;
		  Sd = PlotDesired(Sd,theta,Sdesired,weight,plotparameters,...
				   sysP.Ts,gamma,xspec);
		end
		[x,y,interactiveflag] = ginput(1);
	      end
	      flag = input('Do you really want to use your new sensitivity? (y)',...
			     's');
	      if strcmp(flag,'y') | isempty(flag)
		Sdesired = Sd.Sdesiredmod;
		weight = Sd.weightmod;
	      end
	    end
	    %
	    %   Check that input compatible
	    %
	    if not(length(Sdesired) == length(theta))
	      disp('Incompatible input given!'), pause
	      idx = 1:min(length(Sdesired),length(theta));
	      theta = theta(idx(:));
	      Sdesired = Sdesired(idx(:));
	    end
	    if not(length(Sdesired) == length(weight))
	      weight = ones(size(Sdesired));
	    end
	  else
	    Dflag = 0;
	  end
	  clear Sd;
	end
       
       %
       %    Modify additional parameters
       %
       case 'Other Design Parameters'
	Oflag = 1;
	while Oflag           
	  Sd = PlotInteractive(...
	      theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
	  options = { 'Change upper modulus bound of S',...
		      'Change additional specified values',...
		      'Return to Input Data Menu' };
	  chosenoption = DrawMenu('Other Design Parameters',options);
	  switch chosenoption
	   case 'Change upper modulus bound of S'
	    gamma = NumericalInput(...
		'Give desired upper modulus bound of S:',...
		gamma);
	    
	   case 'Change additional specified values'
	    optionsA = { 'Modify one specification',...
			 'Add specification manually',...
			 'Add specification from current S',... 
			 'Add specification in plot',...
			 'Remove specifications',...
			 'Return to Other Design Parameters' };
	    Aflag = 1;
	%%% Ryozo introduced xspectemp.freq in addition to xspec.freq 
	%%% xspec.freq \in [0,pi] (for computataion)
        %%% xspectemp.freq \in [0,pi]/Ts (for users)
	    while Aflag
	      clc
	      disp('Your current specifications are')
	      disp('Number Frequency  Magnitude   Phase  Sharpness')
	      for k = 1:length(xspec.sharpness)
		tmp.str1 = num2str(xspectemp.freq(k),3);
		tmp.str2 = num2str(xspec.mag(k),3);
		tmp.str3 = num2str(xspec.phase(k),3);
		tmp.str4 = num2str(xspec.sharpness(k),3);
		disp([ blanks(3) int2str(k)...
		       blanks(12-length(tmp.str1)) tmp.str1...
		       blanks(8-length(tmp.str2)) tmp.str2...
		       blanks(10-length(tmp.str3)) tmp.str3...
		       blanks(10-length(tmp.str4)) tmp.str4 ])
		if all(xspectemp.freq(k) ~= [0 pi])
		  tmp.str1 = num2str(-xspectemp.freq(k),3);
		  tmp.str3 = num2str(-xspec.phase(k),3);
		  disp([ blanks(4) ...
			 blanks(12-length(tmp.str1)) tmp.str1...
			 blanks(8-length(tmp.str2)) tmp.str2...
			 blanks(10-length(tmp.str3)) tmp.str3...
			 blanks(10-length(tmp.str4)) tmp.str4 ])
		end
	      end
	      chosenoption = DrawMenu('Additional Specifications',optionsA,0);
	      switch chosenoption
	       case 'Modify one specification'
		idx = NumericalInput(...
		    'Which specifications do you want to change (none = 0)?',...
		    0);
		if idx > 0 & idx <= length(xspectemp.freq)
		  %
		  %  Valid input
		  %
		  xspectemp.freq(idx) = NumericalInput('Give frequency:',...
						       xspectemp.freq(idx));
		  if sysP.Ts ~= 0
		    xspec.freq = xspectemp.freq * sysP.Ts;
		  else
		    xspec.freq = xspectemp.freq;
		  end
    
		  xspec.mag(idx)= NumericalInput('Give magnitude (dB):',...
						 xspec.mag(idx));          
		  xspec.phase(idx) = NumericalInput('Give phase (deg):',...
		    xspec.phase(idx));
		  xspec.sharpness(idx)= NumericalInput(['Give sharpness' ...
		    ' [0,1]:'],xspec.sharpness(idx));	  
		end
		Sd = PlotInteractive(...
		    theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
 		
	       case 'Add specification manually'
		clear tmp
		xspectemp.freq(end+1) = NumericalInput('Give frequency:',0);
		if sysP.Ts ~= 0
		  xspec.freq = xspectemp.freq * sysP.Ts;
		else
		  xspec.freq = xspectemp.freq;
		end  
		xspec.mag(end+1)= NumericalInput('Give magnitude (dB):',0);        
		xspec.phase(end+1) = NumericalInput('Give phase (deg):',0);
		xspec.sharpness(end+1)= NumericalInput(['Give sharpness' ...
		    ' [0,1]:'],1);
		Sd = PlotInteractive(...
		    theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
        
 	       case 'Add specification from current S'
		if isempty(sysS)
		  disp('No current S'), pause
		else
		  clear tmp
		  xspectemp.freq(end+1) = NumericalInput('Give frequency:',0);
		  [numStmp,denStmp] = tfdata(sysS,'v');        
		  if sysP.Ts ~= 0
		    % discrete-time
		    xspec.freq(end+1) = xspectemp.freq(end) * sysP.Ts; 
		    Stmp = polyval(numStmp,exp(i*xspec.freq(end)))/...
			   polyval(denStmp,exp(i*xspec.freq(end)));
		  else
		    % continuous-time
		    xspec.freq(end+1) = xspectemp.freq(end); 
		    Stmp = polyval(numStmp,xspec.freq(end)*i)/...
			   polyval(denStmp,xspec.freq(end)*i);          
		  end 
		  xspec.mag(end+1) = 20*log10(abs(Stmp));
		  xspec.phase(end+1) = angle(Stmp)*180/pi;        
		  xspec.sharpness(end+1)= 1;
		end
		Sd = PlotInteractive(...
		    theta,Sdesired,weight,plotparameters,sysP.Ts, ...
		    gamma,xspec);
		
	       case 'Add specification in plot'
		clear tmp
		disp(['Click in subplot 1 of Figure 1 to give magnitude' ...
		      ' at a certain frequency'])
		[freq1,mag,interactiveflag] = ginput(1);
		while gca ~= Sd.subplot1
		  disp('You should click in subplot 1! Try again!')
		  [freq1,mag,interactiveflag] = ginput(1);
		end
		disp(['Click in subplot 2 of Figure 1 to give the phase' ...
		      ' at the same frequency'])
		[freq2,phase,interactiveflag] = ginput(1);
		while gca ~= Sd.subplot2
		  disp('You should click in subplot 2! Try again!')
		  [freq2,phase,interactiveflag] = ginput(1);
		end
		xspectemp.freq(end+1) = (freq1+freq2)/2;
		if isct(sysP)
		  xspec.freq = xspectemp.freq;
		else
		  xspec.freq = xspectemp.freq * sysP.Ts;
		end
		xspec.sharpness(end+1) = 1;
		xspec.phase(end+1) = phase;
		xspec.mag(end+1) = mag;
		Sd = PlotInteractive(...
		    theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
        
	       case 'Remove specifications'
		idx = NumericalInput(...
		    'Which do you want to remove?',...
		    []);
		if not(isempty(idx))
		  xspectemp.freq(idx) = [];
		  if isct(sysP)
		    xspec.freq = xspectemp.freq;
		  else
		    xspec.freq = xspectemp.freq * sysP.Ts;
		  end
		  xspec.sharpness(idx) = [];
		  xspec.phase(idx) = [];
		  xspec.mag(idx) = [];
		end
		Sd = PlotInteractive(...
		    theta,Sdesired,weight,plotparameters,sysP.Ts,gamma,xspec);
		
	       case 'Return to Other Design Parameters'
		Aflag = 0;
          
	      end
	    end
	   	    
	   case 'Return to Input Data Menu'
	    Oflag = 0;
	  end
	end
	
       %
       %    Default option - returning
       %
       case 'Return to Main Menu'
	inputflag = 1;
      end
      plotparameters.plotvec = { 'loglog', 'semilogx' };      
    end
  
    %
    %   Set additional frequencies for computation
    %
    if sysP.Ts ~= 0
      xspec.freq = xspectemp.freq * sysP.Ts;
    else
      xspec.freq = xspectemp.freq;
    end
    
   %
   %    Controller design menu
   %
   case 'Design Controller'
    %
    %    Detection of no plant 
    %
    if isempty(sysP)
      disp('Input a plant transfer function!')
      disp('Press any key to continue')
      pause
    else
      
    clc
    if sysP.Ts ~=0
      %
      % Discrete-time case
      %
      zvec = exp(i*theta); 
      tmp.fixpoint = xspec.sharpness.*exp(i*xspec.freq);
      tmp.fixval = 10.^(xspec.mag/20).*exp(-i*xspec.phase*pi/180);
    else
      %
      % Continuous-time case
      %
      zvec = (theta*i + 1) ./ (-theta*i + 1);
      tmp.fixpoint = xspec.sharpness.*(i*xspec.freq+1)./(-i*xspec.freq+1);      
      tmp.fixval = 10.^(xspec.mag/20).*exp(-i*xspec.phase*pi/180);       
    end    
    for j = 1:length(tmp.fixpoint)
      if abs(imag(tmp.fixpoint(j))) > 1e-10 
	tmp.fixpoint = [tmp.fixpoint conj(tmp.fixpoint(j))];
	tmp.fixval = [tmp.fixval conj(tmp.fixval(j))];
      else
	tmp.fixpoint(j) = real(tmp.fixpoint(j));
	tmp.fixval(j) = real(tmp.fixval(j));
      end
    end
        
    %
    %   Get transformation matrix K and an initial solution.
    %
    [A,B,K,alpha0,rho0,PCintegrators,exitflag] =...
	SISO2K(sysP,1,gamma,Creldegree,Cintegrators,...
	       tmp.fixpoint,tmp.fixval,1);
    
    %
    %   Run NLSsolver if possible
    %
    if norm(K-eye(size(K))) < 1e-12
      disp('Add additional interpolation!')
      pause
    elseif not(exitflag)
      disp('Controller construction failed!')
      pause
    else
      %
      %   Initialization
      %
      InitOptions = { 'Latest solution',...
		      'Approximate peak solution',...
		      'Spectral zeros',...
		      'Maximum entropy solution',...
		    };
      chosenInitOption = DrawMenu('Initialization of NLSsolver', ...
				  InitOptions,[],1);
      switch chosenInitOption
	
       case 'Latest solution'
	if isempty(rho)
	  disp('No previous solution available'), pause
	  rho = AddZeros(rho0,size(K,1));
	else
	  rho = AddZeros(rho,size(K,1));
	end
	
       case 'Approximate peak solution'
	%
	%   Default values
	%
	if isempty(sharpness)
	  tmp.idx = find(Sdesired == max(Sdesired));
	  if isct(sysP)
	    peakfreq = theta(tmp.idx(1));
	  else
	    peakfreq = theta(tmp.idx(1))/sysP.Ts;
	  end
	  sharpness = 0.95;
	end
	peakfreq = NumericalInput(...
	    'Give approximate peak location:',...
	    peakfreq);
	sharpness = NumericalInput(...
	    'Give sharpness of peak (0,1):',...
	    sharpness);
	if sharpness >= 1 | sharpness < 0
	  disp('The sharpness should be between 0 and 1')
	  sharpness = 0.95;
	end
	  
	if isdt(sysP) 
	  tmp.spectralzeros = sharpness*exp(i*peakfreq*sysP.Ts);
	else
	  tmp.spectralzeros = sharpness*(peakfreq*i+1)./(-peakfreq*i+1);
	end
	for j = 1:length(tmp.spectralzeros)
	  if abs(imag(tmp.spectralzeros(j))) > 1e-10
	    tmp.spectralzeros = [tmp.spectralzeros...
		    conj(tmp.spectralzeros(j))];
	  end
	end
	rho = AddZeros(poly(tmp.spectralzeros),size(K,1));

       case 'Spectral zeros'
	disp('Your current spectral zeros are:')
	disp(spectralzeros')
	spectralzeros = NumericalInput(...
	    ['Give desired initial spectral zeros (length '...
	     num2str(OrderCount(sysP,Creldegree,Cintegrators,xspectemp.freq))...
	     '):'],spectralzeros);
	rho = AddZeros(poly(spectralzeros),size(K,1));
       
       case 'Maximum entropy solution'
	rho = AddZeros(rho0,size(K,1));
      end
            
      %
      %   Initialize variables
      %
      alpha0 = AddZeros(alpha0,size(K,1));
      rho0 = AddZeros(rho0,size(K,1));
      gIK = gamma*eye(size(K)) + K;
      parameters.alpha0 = (gamma*eye(size(K)) + K)*alpha0;
      parameters.disspoly0 = real(GetDissPoly(rho0));
      parameters.K = (gamma*eye(size(K)) - K) / gIK;
      
      %
      %   Determine initial point
      %
      parameters.disspoly = GetDissPoly(rho);
      [alphahat,betahat] = homotopy(parameters,0);
      alpha0 = gIK \ alphahat;
      rho0 = rho;
      
      %
      %    Run the optimization routine
      %
      OptOptions = { 'Choose a particular routine',...
		     'Run all available routines',...
		     'Run all and use the best routine',...
		     'Compare computational performance',...
		   };
      chosenOptOption = DrawMenu('Optimization Options',OptOptions,[],1);
      fixspeczeros = NumericalInput(['Give additional spectral' ...
		    ' zeros in transformed domain:'],[]);
      options = { 'Method 1 (Levenberg-Marquardt)',...
		  'Method 2 (Gauss-Newton)',...
		  'Method 3 (Levenberg-Marquardt) direct',...
		  'Method 4 (Gauss-Newton) direct',...
		};
      switch chosenOptOption
       case 'Choose a particular routine'
	chosenoption = DrawMenu('Design Controller',options,[],1);
	switch chosenoption
	 case 'Method 1 (Levenberg-Marquardt)'
	  [alpha,rho,exitflag,error] = NLSsolverred(...
	      K,gamma,Sdesired(:),zvec(:),alpha0,...
	      rho0(1:(end-length(fixspeczeros))),weight(:), ...
	      'iter',1,fixspeczeros);
	 case 'Method 2 (Gauss-Newton)'
	  [alpha,rho,exitflag,error] = NLSsolverred(...
	      K,gamma,Sdesired(:),zvec(:),alpha0,...
	      rho0(1:(end-length(fixspeczeros))),weight(:), ...
	      'iter',2,fixspeczeros);
	 case 'Method 3 (Levenberg-Marquardt) direct'
	  [alpha,rho,exitflag,error] = NLSsolverred(...
	      K,gamma,Sdesired(:),zvec(:),alpha0, ...
	      rho0(1:(end-length(fixspeczeros))),weight(:), ...
	      'iter',5,fixspeczeros);
	 case 'Method 4 (Gauss-Newton) direct'
	  [alpha,rho,exitflag,error] = NLSsolverred(...
	      K,gamma,Sdesired(:),zvec(:),alpha0, ...
	      rho0(1:(end-length(fixspeczeros))),weight(:), ...
	      'iter',6,fixspeczeros);
	end
	
       case 'Run all available routines'
	kvec = [ 1 2 5 6 ];
	for k = kvec
	  t = cputime;
	  [alpha,rho,exitflag,error] = NLSsolverred(...
	      K,gamma,Sdesired(:),zvec(:),alpha0, ...
	      rho0(1:(end-length(fixspeczeros))),weight(:), ...
	      'iter',k,fixspeczeros);
	  alphalogg(:,k) = alpha;
	  loggtime(k) = cputime - t;
	  loggerror(k) = error;
	end
	disp('******************************************')
	for k = kvec
	  disp([ options{find(kvec == k)}...
		 blanks(50 - length(options{find(kvec == k)}))...
		 ' Computation time : '...
		 blanks(5-length(num2str(loggtime(k))))...
		 num2str(loggtime(k))...
		 '  error: '...
		 blanks(8-length(num2str(loggerror(k))))...
		 num2str(loggerror(k))])
	end
	disp('******************************************')
	
       case 'Run all and use the best routine'
	kvec = [ 1 2 5 6 ];
	for k = kvec
	  t = cputime;
	  [alpha(:,k),rho,exitflag,error] = NLSsolverred(...
	      K,gamma,Sdesired(:),zvec(:),alpha0, ...
	      rho0(1:(end-length(fixspeczeros))),weight(:), ...
	      'iter',k,fixspeczeros);
	  loggtime(k) = cputime - t;
	  loggerror(k) = error;
	end
	idx = find(loggerror == min(loggerror));
	disp([ options{idx(1)} ' chosen'])
	alpha = alpha(:,idx(1));
	
       case 'Compare computational performance'
	chosenoptions = DrawMenu('Design Controller',options,...
				 [],1,1);
	% Load weight matrix from file
	dirname = 'WeightMatrixFiles';
	files = dir([dirname '/*.mat']);
	availfiles = {};
	for k = 1:length(files)
	  availfiles{k} = files(k).name;
	end
	chosenfile = DrawMenu('File containing weight matrix', ...
			      availfiles);
	if isstr(chosenoptions)
	  k = 1;
	  while not(strcmp(options{k},chosenoptions))
	    k = k + 1;
	  end
	  chosenoptions = k
	end
	load([dirname '/' chosenfile],'weightmat')	 
	% Normalize weightmat
	weightmat = weightmat ./...
	    (ones(size(weightmat,1),1)*mean(weightmat));
	for k = 1:length(chosenoptions)
	  for l = 1:size(weightmat,2)
	    t = cputime;
	    [alpha(:,k),rho,exitflag,error] = NLSsolverred(...
		K,gamma,Sdesired(:),zvec(:),alpha0, ...
		rho0(1:(end-length(fixspeczeros))),weightmat(:,l), ...
		'iter',chosenoptions(k),fixspeczeros);
	    loggtimemat(k,l) = cputime - t;
	    loggerrormat(k,l) = error;
	  end
	end
	alpha = alpha(:,1);
	%	 %  Normalize errors
	%	 loggerrormat = loggerrormat ./ ...
	%	     (ones(length(chosenoptions),1) * min(loggerrormat));
	loggerrormean = mean(loggerrormat')';
	loggerrorstd = std(loggerrormat')';
	loggtime = mean(loggtimemat')';
	disp('******************************************')
	for k = 1:length(chosenoptions)
	  disp(options{chosenoptions(k)})
	  disp([blanks(5) 'Computation time: '...
		blanks(8-length(num2str(loggtime(k))))...
		num2str(loggtime(k))])
	  disp([blanks(5) 'Mean error: '...
		blanks(14-length(num2str(loggerrormean(k))))...
		num2str(loggerrormean(k))])
	  disp([blanks(5) 'Std dev error: '...
		blanks(11-length(num2str(loggerrorstd(k))))...
		num2str(loggerrorstd(k))])
	end
	disp('******************************************')       
	clear loggerrormat loggerrormean loggerrorstd...
	    loggtimemat chosenoptions
      end
      clear OptOptions chosenOptOption
      
      %
      %   Recover solution
      %
      [sysC,numC,denC,sysS] = Alpha2sysC(alpha,K,sysP,PCintegrators, ...
					 tmp,Creldegree,Cintegrators);
      sysCvec{1} = sysC;
      sysPCvec{1} = sysP*sysC; 
      sysSvec{1} = minreal(feedback(1,sysPCvec{1}));
      sysTvec{1} = parallel(1,-sysSvec{1});
      clear tmp
      
      %
      %    Leave the output from the optimization routine
      %
      disp('You have sucessfully designed a controller')
      disp('Press any key to continue')
      pause
    end
    
    end
    
   %
   %   Plotting the sensitivity function, step responses etc
   %
   case 'Plot Menu'
    %
    %    Detection of no controller
    %
    if isempty(theta) | isempty(sysCvec{1})
      disp('Design a controller first!')
      disp('Press any key to continue')
      pause
    else
       
      plotoptions = { 'Plot Bode of S',...
		      'Plot Bode of T',...
		      'Plot Bode of CS',...
		      'Plot Bode of PS',...
		      'Plot Bode of PC',...
		      'Plot Bode of Plant',...
		      'Plot Bode of Controller',...
		      'Plot Weight',...
		      'Plot Step Response',...
		      'Plot Control Input for Step Response',...
		      'Change Plot Options',...
		      'Return to Main Menu' };
      
      %
      %   Plotting loop
      %
      plotparameters.lower = theta(1);
      plotparameters.upper = theta(end);
      if sysP.Ts ~= 0
        plotparameters.lower = plotparameters.lower/sysP.Ts;
        plotparameters.upper = plotparameters.upper/sysP.Ts;
      end
      plotflag = 0;
      while not(plotflag)
	clc
	if isempty(sysCvec{1})
	  disp('You have no feasible controller design')
	else
	  disp('Designed controller:')
	  sysCvec{1}
	  disp('Corresponding sensitivity function:')
	  sysSvec{1}
	end
	chosenplotoption = DrawMenu('Plot Menu',plotoptions,0);
	
	switch chosenplotoption
	  
	 %
	 %   Plot sensitivity function   
	 %
	 case 'Plot Bode of S'
	  figure(2), clf
	  PlotFunction('sensitivity',sysSvec,...
		       'Bode Plot of Sensitivity Function',...
		       plotparameters,theta,Sdesired);
	 case 'Plot Bode of T'
	  figure(3), clf
	  PlotFunction('bode',sysTvec,...
		     'Bode Plot of Complementary Sensitivity Function',...
		       plotparameters,theta); 
	 case 'Plot Bode of CS'
	  figure(4), clf
	  for k = 1:length(sysCvec)
	  sys{k} = series(sysCvec{k},sysSvec{k});
	  end
	  PlotFunction('bode',sys,...
		       'Bode Plot of CS',...
		       plotparameters,theta);
	 case 'Plot Bode of PS'
	  figure(5), clf
	  for k = 1:length(sysSvec)
	    sys{k} = series(sysP,sysSvec{k});
	  end
	  PlotFunction('bode',sys,...
		       'Bode Plot of PS',...
		       plotparameters,theta); 
	 case 'Plot Bode of PC'
	  figure(6), clf
	  for k = 1:length(sysSvec)
	    sys{k} = series(sysP,sysCvec{k});
	  end
	  PlotFunction('bode',sys,...
		       'Bode Plot of PC',...
		       plotparameters,theta);
	 case 'Plot Bode of Plant'
	  figure(7), clf
	  bode(sysP,plotparameters.linestylevec{1})
	  eval(plotparameters.grid);
	  title('Plot Bode of Plant')
	  ChangeTextStyle(gcf,plotparameters.myfontsize,...
			  plotparameters.myfontweight,...
			  plotparameters.mylinewidth);
	 case 'Plot Bode of Controller'
	  figure(8), clf
	  PlotFunction('bode',sysCvec,...
		       'Bode Plot of Controller',...
		       plotparameters,theta); % Ryozo added theta. July 7, 2004
	 case 'Plot Weight'
	  figure(9), clf
	  if sysP.Ts ~= 0 
	    semilogy(theta,weight,'kx')
	    axis([theta(1) theta(end) axisvec(3:4)]);
	  else
	    loglog(theta,weight,'kx')
	    axisvec = axis;
	    axis([plotparameters.lower plotparameters.upper ...
		  axisvec(3:4)]);
	    clear axisvec
	  end
	  eval(plotparameters.grid);
	  title('Weight')
	  ylabel('Magnitude')
	  xlabel('Frequency (rad/sec)')
	  ChangeTextStyle(gcf,plotparameters.myfontsize,...
			  plotparameters.myfontweight,...
			  plotparameters.mylinewidth);
	 case 'Plot Step Response'
	  figure(10), clf
	  PlotFunction('step',sysTvec,...
		       'Step Response',...
		       plotparameters);
	 case 'Plot Control Input for Step Response'
	  figure(11), clf
	  for k = 1:length(sysSvec)
	    sys{k} = feedback(sysCvec{k},sysP);
	  end
	  PlotFunction('step',sys,...
		       'Control Input for Step Response',...
		       plotparameters);
	 case 'Change Plot Options'
	  plotparameters.legendvec{1} = StringInput(...
	      'Give Example Name for Display',plotparameters.legendvec{1});
	  
	  plotparameters.T =  NumericalInput('Give length of step response:',...
					     plotparameters.T);
	  flag = input('Gridding on? (y) ','s');
	  if strcmp(flag,'y') | isempty(flag)
	    plotparameters.grid = 'grid on';
	  else
	    plotparameters.grid = 'grid off';
	  end
	  plotparameters.lower = NumericalInput(...
	      'Give lower frequency for plotting:',...
	      plotparameters.lower);
	  plotparameters.upper = NumericalInput(...
	      'Give upper frequency for plotting:',...
	      plotparameters.upper);
	  plotparameters.myfontsize = NumericalInput(...
	      'Give font size:',...
	      plotparameters.myfontsize);
	  flag = input('Font weight, bold or normal? (bold) ','s');
	  if strcmp(flag,'bold') | isempty(flag)
	    plotparameters.myfontweight = 'bold';
	  else
	    plotparameters.myfontweight = 'normal';
	  end
	  plotparameters.mylinewidth = NumericalInput(...
	      'Give line width:',...
	      plotparameters.mylinewidth);
	 case 'Return to Main Menu'
	  plotflag = 1;
	end
      end    
    end 
    
   %
   %   Printing the plots
   %
   case 'Print Menu'
    printoptions = { 'Print Specific Plot',...
		     'Print All Default',...
		     'Change Printing Parameters',...
		     'Return to Main Menu' };
    
    %
    %   Printing loop
    %
    printflag = 0;
    while not(printflag)
      clc
      chosenprintoption = DrawMenu('Print Menu',printoptions,0);
      
      switch chosenprintoption
       
       %
       %   Print sensitivity function   
       %
       case  'Print Specific Plot'
	plotno =  NumericalInput('Give plot number:',1);
	if plotno > 11
	  disp('Given number smaller than 10')
	  plotno = 1;
	end
	PrintFunction(plotno,printparameters);
       case 'Print All Default'
	for k = 1:11 
	  PrintFunction(k,printparameters);
	end
       case 'Change Printing Parameters'
	printparameters.examplename = StringInput('Give example name',...
						  printparameters.examplename);
	printparameters.type = StringInput('Give file type',...
					   printparameters.type,...
					   { 'eps', 'jpeg'});
	printparameters.width =  NumericalInput(...
	    'Give plot width:',...
	    printparameters.width);
	printparameters.height =  NumericalInput(...
	    'Give plot height:',...
	    printparameters.height);
       case 'Return to Main Menu'
	printflag = 1;
      end
    end    
    
   %
   %    Load a controller design elsewhere (to compare performance)
   %
   case 'Load Controller for Comparison'
    files = dir;
    dirname = {};
    for k = 1:length(files)
      if files(k).isdir
	dirname{end+1} = files(k).name;
      end
    end
    dirname = DrawMenu('Available directories',dirname);       
    files = dir([dirname '/*.mat']);
    clear availfiles
    for k = 1:length(files)
      availfiles{k} = files(k).name;
    end
    availfiles{end+1} = 'Return to Main Menu';
    chosenfile = DrawMenu('Available files in directory',availfiles);
    if not(strcmp(chosenfile,'Return to Main Menu'))
      load([dirname '/' chosenfile])   
      if isempty(sysCmanual)
 	disp('The chosen file didn''t contain a variable sysCmanual')
	sysCvec{2} = tf(1);
	pause(3)
      else
	sysCvec{2} = sysCmanual; 
      end
      sysCvec{2} = sysCmanual;
      sysPCvec{2} = series(sysCvec{2},sysP);
      sysSvec{2} = minreal(feedback(1,sysPCvec{2}),1e-6);
      sysTvec{2} = parallel(1,-sysSvec{2});
      plotparameters.legendvec{2} = 'Previous design'; 
    end
   
   %
   %    Load a previous design from a mat-file
   %
   case 'Load Design'
    files = dir;
    dirname = {};
    for k = 1:length(files)
      if files(k).isdir
	dirname{end+1} = files(k).name;
      end
    end
    dirname = DrawMenu('Available directories',dirname);
    files = dir([dirname '/*.mat']);
    availfiles = {};
    for k = 1:length(files)
      availfiles{k} = files(k).name;
    end
    availfiles{end+1} = 'Return to Main Menu';
    chosenfile = DrawMenu('Available files in directory',availfiles);
    if not(strcmp(chosenfile,'Return to Main Menu'))
     load([dirname '/' chosenfile])
    end
    mainflag = 0;
   
   %
   %    Save the current variables in a mat-file
   %
   case 'Save Design'
    files = dir;
    dirname = {};
    for k = 1:length(files)
      if files(k).isdir
	dirname{end+1} = files(k).name;
      end
    end
    dirname{end+1} = 'Make new directory';
    dirname = DrawMenu('Available directories',dirname);
    if strcmp(dirname,'Make new directory')
      dirname = StringInput('Give directory: ','DesignMatFiles');
    end
    if not(isdir(dirname))
      mkdir(dirname);
    end
    filename = input('Give filename (empty doesn''t save): ','s');
    if not(isempty(filename))
      save([dirname '/' filename '.mat'])
    end
   
   %
   %    Display variables
   %
   case 'Display Variables'
    clc
    disp('Currently')
    who
    name = input(['Which do you want to display (enter returns to' ...
		  ' main menu)? '],'s');
    while not(isempty(name)) 
      try
	disp(' ')
	eval(name), pause
	disp(' ')
      catch
	disp([ name ' is not in the workspace']), pause
      end
      name = input(['Which do you want to display (enter returns to' ...
		  ' main menu)? '],'s');
    end
    
   %
   %    Clear some or all variables
   %
   case 'Clear Variables'
    
    %
    %   Clear loop
    %
    clearflag = 0;
    while not(clearflag)
      clc
      clearoptions = { 'Clear All Variables in the Workspace',...
		       'Clear Some Variables in the Workspace',...
		       'Return to Main Menu' };
      chosenclearoption = DrawMenu('Clear Menu',clearoptions,0);
      
      switch chosenclearoption
       
       case 'Clear All Variables in the Workspace'
        flag = input('Do you really want to clear all the variables? (y)','s');
        if strcmp(flag,'y') | isempty(flag)
	  clear all
            clearflag = 1;
            mainflag = 0;
        end             
       case 'Clear Some Variables in the Workspace'
	disp('Currently')
	who
	name = input('Which do you want to clear? ','s');
	try
	  eval(name);
	  clear(name);
	catch
	  disp([ name ' is not available in the workspace']), pause
	end
       case 'Return to Main Menu'
	clearflag = 1;
      end
    end    

   %
   %    Close some or all figures
   %
   case 'Close Figures'
    
    %
    %   Close figures loop
    %
    closeflag = 0;
    while not(closeflag)
      clc
      closeoptions = { 'Close All Figures',...
		       'Close A Figure',...
		       'Return to Main Menu' };
      chosencloseoption = DrawMenu('Close Figure Menu',closeoptions,0);
      
      switch chosencloseoption
	
       case 'Close All Figures'
	flag = input('Do you really want to close all figures? (y)','s');
	if strcmp(flag,'y') | isempty(flag)
          close all hidden
          closeflag = 1;
          mainflag = 0;         
	end          
       case 'Close A Figure'
	name = input('Which figure do you want to close? '); 
	figure(name),
	flag = input('Do you really want to close this figure? (y)','s');
	if strcmp(flag,'y') | isempty(flag)
          close
	end
       case 'Return to Main Menu'
	closeflag = 1;
      end
    end   

   %
   %   Leave the program
   %
   case 'Exit'
    mainflag = 1;  
  end
end

%
%   Save the workspace in all cases
%
save DesignMatFiles/DataLatest.mat
