%
%   This function solves the sub problem of the trust region problem
%
%   Input Gk - Hessian
%         gk - gradien
%         dk - trusted region
%
function p = trustdirect(Gk, gk, dk)

%
%   Termination parameters
%
maxiter = 500;
sigma1 = 1e-12;

[R,stat] = chol(Gk);
if (stat == 0)
    p = R \ (R' \ (-1*gk));
    if (norm(p) < dk)
        return
    end
end
%
%   Now we know that the solution is on the boundary of the domain
%    so we try to solve the non-linear equation
%
%         1         1
%      -------  = ----
%      || p ||     dk
%
%   We will apply Newton's method using the algorithm (3.14) of
%    More et al.
%
%   Initialize variables
%
n = size(gk,1);
I = eye(n);
lmbS = max(-diag(Gk));
lmbL = max( [ eps, lmbS, norm(gk)/dk - norm(Gk,1) ] );
lmbU = norm(gk)/dk + norm(Gk,1);
lambda = sqrt(lmbU*lmbL);
p = zeros(n,1);

iter = 0;
while abs(dk - norm(p)) > sigma1*(sigma1+dk) & (norm(p) > dk | lambda ~= 0) ...
  & iter < maxiter %abs(f) > sigma1*dk
    iter = iter + 1; 
  
    %   Determine new lambda candidate
    [R,stat] = chol(Gk + lambda * I);
    if stat == 0 & norm(gk) ~= 0
        %   Perform Newton step
        p = R \ (R' \ (-gk));
        lambda = updatelambda(R,p,dk,lambda);
    else
        lambda = lmbS;
    end
    
    %   Safeguard lambda
    lambda = safeguard(lmbS,lmbL,lmbU,lambda);

    %   Determine new p
    B = Gk + lambda*I;
    [R,stat] = chol(B);
    
    if stat == 0
        p = R \ (R' \ (-gk));
        if norm(p) < dk
%           disp('Hard case!')
           [ zhat, tau ] = getzhatandtau(R,p,dk);
           p = p + tau* zhat;
%	   disp(lambda)
%	   disp(min(eig(Gk)))
       end
    end

    %
    %   Update the lambdalimits
    %
    if stat == 0 & norm(p) < dk
        lmbU = min(lmbU,lambda);
    else
        lmbL = max(lmbL,lambda);
    end

    if stat == 0 & norm(p) < dk
        %   Criteria (3.8)
        lmbS = max(lmbS,lambda - norm(R*zhat)^2);
    elseif stat ~= 0
        %   Criteria (3.9)
        x = R'\B(1:stat-1,stat);
        dirac = -B(stat,stat)+x'*x;
        u = [ -R\x; 1 ];
        lmbS = max(lmbS,lambda + dirac/norm(u)^2);
    end
    
    lmbL = max(lmbL,lmbS);
    
end