%
%   This function solves the set of nonlinear equations
%
%       g(alpha) = 0
%
%   using a homotopy continuation method. The initial point 
%    is taken as the central solution. 
%
%   Input: 
%    parameters - initial point and problem data
%    printflag - print to screen or not (default off)
%
%   Output: 
%    alpha - numerator polynomial
%
function [alpha,beta] = homotopy(parameters,printflag)

if nargin < 2 | isempty(printflag)
    printflag = 0;
end

%
%   Parameters
%
stepdecrease = 1.5;     %  Decrease of step when outside D
stepmu = 0.3;           %  Tolerance in predictor step
maxiter = 100;          %  Maximal number of main iterations  
errorbound = 1e-12;     %  Maximal inf-norm of LHS, final. Changed
                        %   from 1e-8 by Anders, April 26, 2004
errorboundway = 1e-8;   %  Maximal inf-norm of LHS, on the way
digit = 8;              %  Number of digits to display
displayrate = 1;        %  How often to display

%
%    Initial point & initialize vectors
%
disspoly0 = parameters.disspoly0;
alpha = parameters.alpha0;
disspoly1 = parameters.disspoly;
K = parameters.K;

%
%   Normalize dissipation polynomial
%
kappa = disspoly0(1) / disspoly1(1);
disspoly1 = disspoly1 * kappa;

%
%   Check whether we are looking for the central solution
%
if norm(disspoly1-disspoly0) < errorbound^2
   if printflag
     disp('This is the central solution.')
   end
   error = EvaluateG(alpha,K,disspoly0,disspoly1,1);
%   if max(abs(error)) > errorbound,
%       disp('Solution might be inaccurate')
%       disp(error)
%   end
   if nargout > 1
     beta = K*alpha;
   end
   return
end

%
%   Gradient at the central solution
%
jacG = GetJacobian(alpha,K);

%
%   Compute dG/dnu
%
dG_dnu = disspoly0 - disspoly1;

if printflag
    disp('------------------------------------------------------')
    disp('    Nevanlinna-Pick solver by nonlinear equations     ')
    disp('------------------------------------------------------')
    disp('                    Corrector step                    ')
    disp('     nu       iteration      equation error           ')
    disp('------------------------------------------------------')
end

nu = 0; k = 0;
while nu ~= 1
    k = k + 1;
    
    %
    %   Predictor step:
    %
    %    Calculate the direction for a step in nu
    %
    dalpha_dnu = jacG \ dG_dnu;
    
    %
    %   Determine the step size for nu
    %
    dnu = FindStepSize(alpha,dalpha_dnu,K,stepmu);
    if nu + dnu > 1,
        dnu = 1 - nu;
    end
    
    %
    %   We need to stay within D
    %
    itertmp = 0;
    while IsInD(K,alpha-dnu*dalpha_dnu) ~= 1
        dnu = dnu / stepdecrease;
        itertmp = itertmp + 1;
        if itertmp > maxiter
            disp('Broken in D test')
            break
        end
    end
    nu = nu + dnu;
    if printflag
        disp(num2str(nu,digit))
    end

    %
    %   Corrector step:
    %
    %       Update the polynomial alpha check error.
    %
    alpha = alpha - dnu*dalpha_dnu;
    
    iter = 0;
    errorold = 0;
    jacG = GetJacobian(alpha,K);
    Gvalue = EvaluateG(alpha,K,disspoly0,disspoly1,nu);
    
    while (nu < 1 | max(abs(Gvalue)) > errorbound) & ...
	  (nu == 1 | max(abs(Gvalue)) > errorboundway)
        dalpha = - jacG \ Gvalue;
	
	%
	%   Fix, Anders May 11, 2004
	% 
	if any(isinf(dalpha))
	  dalpha = - (jacG+eps*eye(size(jacG))) \ Gvalue;
	end
        
        %
        %   We check that we stay in D.
        %
        itertmp = 0;
        lambda = 1;
        while IsInD(K,alpha+lambda*dalpha) ~= 1     
            lambda = lambda / stepdecrease;
            itertmp = itertmp + 1;
            if itertmp > maxiter
                disp('Broken in D test')
                break
            end
	end
	
        %
        %   Update the solution and check interpolation error
        %
        alpha = alpha + lambda*dalpha;
        jacG = GetJacobian(alpha,K);
        Gvalue = EvaluateG(alpha,K,disspoly0,disspoly1,nu);
        error = max(abs(Gvalue));
	  
        iter = iter + 1;
        if printflag & mod(iter,displayrate) == 0

            errorold = error; 
            
            iterstr = num2str(iter);
            errorstr = num2str(error,digit);
            disp([blanks(20 - length(iterstr)) iterstr, ...
                        blanks(20 - length(errorstr)) errorstr])
        end
        if iter > maxiter
            disp('Broken corrector step')
            break
        end
    end
end

if printflag
    disp('--------------------------------------------------')
end

%
%   Scale back and give output
%
alpha = alpha / sqrt(kappa);
if nargout > 1
  beta = K*alpha;
end


%
%   A local function for evaluating the LHS of the nonlinear equation
%
function G = EvaluateG(alpha,K,disspoly0,disspoly1,nu)

G = GetS(alpha)*K*alpha - (1-nu)*disspoly0 - nu*disspoly1;